@@ -30,6 +30,10 @@
     http://www2.lm-sensors.nu/~lm78/download.html
 */
 
+/*   25th March 2004
+     Support for Sis655 chipsets added by Ken Healy
+*/
+
 /*
     Note: we assume there can only be one SiS645 with one SMBus interface
 */
@@ -71,6 +75,11 @@
 #define PCI_DEVICE_ID_SI_651 0x0651
 #endif
 
+/* SiS655 north bridge (defined in 2.4.22)*/
+#ifndef PCI_DEVICE_ID_SI_655
+#define PCI_DEVICE_ID_SI_655 0x0655
+#endif
+
 /* SiS746 north bridge (defined in 2.4.21) */
 #ifndef PCI_DEVICE_ID_SI_746
 #define PCI_DEVICE_ID_SI_746 0x0746
@@ -133,6 +142,7 @@
 #define SIS645_PROC_CALL  0x04
 #define SIS645_BLOCK_DATA 0x05
 
+static struct pci_driver sis645_driver;
 static struct i2c_adapter sis645_adapter;
 static u16 sis645_smbus_base = 0;
 
@@ -260,6 +270,8 @@ static int __devinit sis645_hotplug_smbus(void)
 				PCI_DEVICE_ID_SI_650, NULL))
 			&& (NULL == pci_find_device(PCI_VENDOR_ID_SI,
 				PCI_DEVICE_ID_SI_651, NULL))
+			&& (NULL == pci_find_device(PCI_VENDOR_ID_SI,
+				PCI_DEVICE_ID_SI_655, NULL))
 			&& (NULL == pci_find_device(PCI_VENDOR_ID_SI,
 				PCI_DEVICE_ID_SI_735, NULL))
 			&& (NULL == pci_find_device(PCI_VENDOR_ID_SI,
@@ -448,6 +460,20 @@ static s32 sis645_access(struct i2c_adapter * adap, u16 addr,
 	return 0;
 }
 
+static void sis645_inc(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_INC_USE_COUNT;
+#endif
+}
+
+static void sis645_dec(struct i2c_adapter *adapter)
+{
+#ifdef MODULE
+	MOD_DEC_USE_COUNT;
+#endif
+}
+
 static u32 sis645_func(struct i2c_adapter *adapter)
 {
 	return I2C_FUNC_SMBUS_QUICK | I2C_FUNC_SMBUS_BYTE |
@@ -463,10 +489,10 @@ static struct i2c_algorithm smbus_algorithm = {
 };
 
 static struct i2c_adapter sis645_adapter = {
-	.owner		= THIS_MODULE,
-	.name		= "unset",
 	.id		= I2C_ALGO_SMBUS | I2C_HW_SMBUS_SIS645,
 	.algo		= &smbus_algorithm,
+	.inc_use	= sis645_inc,
+	.dec_use	= sis645_dec,
 };
 
 static struct pci_device_id sis645_ids[] __devinitdata = {
@@ -506,7 +532,8 @@ static int __devinit sis645_probe(struct pci_dev *dev,
 			sis645_smbus_base);
 
 	/* Everything is happy, let's grab the memory and set things up. */
-	if (!request_region(sis645_smbus_base, SMB_IOSIZE, "sis645-smbus")) {
+	if (!request_region(sis645_smbus_base, SMB_IOSIZE,
+			    sis645_driver.name)) {
 		dev_err(dev, "SMBus registers 0x%04x-0x%04x "
 			"already in use!\n", sis645_smbus_base,
 			sis645_smbus_base + SMB_IOSIZE - 1);
