@@ -79,7 +79,7 @@ static struct i2c_driver lm75_driver = {
 
 /* -- SENSORS SYSCTL START -- */
 
-#define LM75_SYSCTL_TEMP 1200	/* Degrees Celcius * 10 */
+#define LM75_SYSCTL_TEMP 1200	/* Degrees Celsius * 10 */
 
 /* -- SENSORS SYSCTL END -- */
 
@@ -94,16 +94,14 @@ static ctl_table lm75_dir_table_template[] = {
 	{0}
 };
 
-static int lm75_id = 0;
-
 static int lm75_attach_adapter(struct i2c_adapter *adapter)
 {
 	return i2c_detect(adapter, &addr_data, lm75_detect);
 }
 
 /* This function is called by i2c_detect */
-int lm75_detect(struct i2c_adapter *adapter, int address,
-		unsigned short flags, int kind)
+static int lm75_detect(struct i2c_adapter *adapter, int address,
+		       unsigned short flags, int kind)
 {
 	int i;
 	struct i2c_client *new_client;
@@ -192,8 +190,6 @@ int lm75_detect(struct i2c_adapter *adapter, int address,
 
 	/* Fill in the remaining client fields and put it into the global list */
 	strcpy(new_client->name, client_name);
-
-	new_client->id = lm75_id++;
 	data->valid = 0;
 	init_MUTEX(&data->update_lock);
 
@@ -259,8 +255,12 @@ static int lm75_write_value(struct i2c_client *client, u8 reg, u16 value)
 
 static void lm75_init_client(struct i2c_client *client)
 {
-	/* Initialize the LM75 chip */
-	lm75_write_value(client, LM75_REG_CONF, 0);
+	int i;
+
+	/* Enable if in shutdown mode */
+	i = lm75_read_value(client, LM75_REG_CONF);
+	if (i >= 0 && (i & 0x01))
+		lm75_write_value(client, LM75_REG_CONF, i & 0xfe);
 }
 
 static void lm75_update_client(struct i2c_client *client)
